"""
Module responsible for providing information regarding switches supported by Cumulus Linux
"""

try:
    from cumulus_platform_info.__init__ import __json_file_path__
except:
    from __init__ import __json_file_path__

import logging
import json
import os
import sys
import subprocess
from pprint import pformat

log = logging.getLogger(__name__)

class CumulusPlatformInfo(object):
    """
    It query json platform files by platform detect type
    if there are multiple platforms under a single platform detect type
    then use decode-syseeprom info to figure out the right one.
    """
    def __init__(self):
        self._platform_detect = None
        self.all_platform_details = dict()
        self.platform_details = dict()
        self.abs_path = os.path.join(sys.prefix, __json_file_path__)
        self.platform_entry = None
        self._decode_syseeprom = dict()
        self.read_thru_platform_files()

    @property
    def platform_detect(self):
        """
        Only grab platform detect the first time. subsequent calls should use
        the cache value.
        :return: platform detect string
        """
        exec_str = '/usr/bin/platform-detect'
        if not self._platform_detect and os.path.exists(exec_str):
            log.debug("platform_detect() running %s" % exec_str)
            self._platform_detect = subprocess.check_output(exec_str).strip()
        return self._platform_detect

    def decode_syseeprom(self):
        """
        Convert the text output from decode syseeprom into a dict.
        Hopefully one day decode-syseeprom will have JSON output
        :return: decode syseeprom output from a cumulus switch
        """
        if self._decode_syseeprom:
            return self._decode_syseeprom

        log.debug("decode_syseeprom() running /usr/cumulus/bin/decode-syseeprom")
        output = subprocess.check_output(
            ['/usr/cumulus/bin/decode-syseeprom'])
        ret = output.splitlines()
        start_on_next_line = False
        for _line in ret:
            if _line.startswith('---'):
                start_on_next_line = True
                continue
            if start_on_next_line:
                chunk = []
                chunk.append(_line[0:21])
                chunk.append(_line[21:28])
                chunk.append(_line[28:30])
                chunk.append(_line[30:])
                if chunk[0].strip() == 'CRC-32':
                    continue
                if chunk[1].strip().startswith('0x'):
                    header = chunk[0].strip().replace(' ', '_').lower()
                    value = chunk[3].strip()
                    self._decode_syseeprom[header] = value
        return self._decode_syseeprom

    def compare_entries_using_vendor(self):
        """
        :return: based on vendor name return valid platform db entry
        Some of the fields in these dictionaries are unicode so convert
        them all to strings via str()
        """
        eeprom_output = self.decode_syseeprom()
        vendor_name = None

        if 'vendor_name' in eeprom_output:
            vendor_name = str(eeprom_output.get('vendor_name').strip().lower())
        elif 'manufacturer' in eeprom_output:
            vendor_name = str(eeprom_output.get('manufacturer').strip().lower())

        if vendor_name:
            platform_db_vendor_name = vendor_name

            if vendor_name.startswith('hewlett-packard'):
                platform_db_vendor_name = 'hp'
            elif vendor_name.startswith('supermicro'):
                platform_db_vendor_name = 'supermicro'
            elif vendor_name.startswith('penguin'):
                platform_db_vendor_name = 'penguin'

            for _entry in self.platform_details:
                if str(_entry.get('vendor').lower()) == platform_db_vendor_name:
                    return _entry

        # Just guess! usually when vendor name is not mentioned.
        # Bad idea but not sure what else to do without access to every
        # single switch device and then manually verify this fix.
        return self.platform_details[0]

    def run(self):
        """
        find the listing for a particular platform detect string.
        if it has more than one entry, then run decode-syseeprom and
        grab the vendor name. if the vendor name is available use that
        to figure out which platform detect entry to match on.
        otherwise, always just return the first entry found
        """
        if not self.platform_entry:
            self.platform_details = self.all_platform_details.get(self.platform_detect)

            if self.platform_details:
                if len(self.platform_details) > 1:
                    self.platform_entry = self.compare_entries_using_vendor()
                else:
                    self.platform_entry = self.platform_details[0]
        return self.platform_entry

    def read_thru_platform_files(self):
        """
        cache all platform details files, from json before making a query
        """
        for _file in os.listdir(self.abs_path):
            filename = "%s/%s" % (self.abs_path, _file)
            json_dict = open(filename).read()
            self.all_platform_details = dict(list(json.loads(json_dict).items()) +
                                             list(self.all_platform_details.items()))
