# pylint: disable=e0611
# pylint: disable=e1101
"""
Print and Analysis Module for Linux bond interfaces
"""

from netshow.cumulus import print_iface as cumulus_print_iface
import netshow.linux.print_bond as linux_print_bond
from tabulate import tabulate


class PrintBondMember(cumulus_print_iface.PrintIface,
                      linux_print_bond.PrintBondMember):
    """
    Print and Analysis Class for Linux bond member interfaces
    """

    def bondmem_details(self):
        """
        :return: string with output shown when netshow interfaces is issued on a \
        bond member
        """
        _header = ['Bond Details', '']
        _master = self.iface.master
        _printbond = PrintBond(_master)
        _table = []
        _table.append(['Master Bond:', _master.name])
        _table.append(['State In Bond:', self.state_in_bond])
        _table.append(['Link Failures:', self.iface.linkfailures])
        _table.append(['Bond Members:', ', '.join(_master.members.keys())])
        _table.append(['Bond Mode:', _printbond.mode])
        _table.append(['Load Balancing:', _printbond.hash_policy])
        _table.append(['Minimum Links:', _master.min_links])
        _lacp_info = self.iface.master.lacp
        if _lacp_info:
            _table.append(['LACP Sys Priority:', _master.lacp.sys_priority])
            _table.append(['LACP Rate:', _printbond.lacp_rate()])
            _table.append(['LACP Rate:', _printbond.lacp_bypass()])

        return tabulate(_table, _header)

    def cli_output(self):
        """
        cli output of the linux bond member interface.
        Does not print out counters for cumulus VM
        :return: output for 'netshow interface <ifacename>'
        """
        _str = self.cli_header() + self.new_line()
        _str += self.bondmem_details() + self.new_line()
        if self.iface.counters.tx:
            _str += cumulus_print_iface.PrintIface.counters_summary(self) + \
                self.new_line()
        _str += self.lldp_details() + self.new_line()
        return _str


class PrintBond(cumulus_print_iface.PrintIface, linux_print_bond.PrintBond):
    """
    Print and Analysis Class for Linux bond interfaces
    """
    def lacp_bypass(self):
        """
        :return print lacp bypass status
        """
        _lacp = self.iface.lacp
        if _lacp:
            if _lacp.bypass == '1':
                return 'LACP Bypass Active'
            elif _lacp.bypass == '0':
                return 'LACP Bypass Inactive'

        return 'LACP Bypass Not Supported'

    def in_clag(self):
        """
        :return: print clag status
        """
        if self.iface.clag_enable == '0':
            return 'CLAG Inactive'
        elif self.iface.clag_enable == '1':
            return 'CLAG Active'
        else:
            return 'CLAG Not Supported'

    def bond_details(self):
        """
        print out table with bond details for netshow interface [ifacename]
        """
        _header = ['Bond Details', '']
        _table = []
        _table.append(['Bond Mode:', self.mode])
        _table.append(['Load Balancing:', self.hash_policy])
        _table.append(['Minimum Links:', self.iface.min_links])
        if self.in_clag:
            _table.append(['In CLAG:', self.in_clag()])
        _lacp_info = self.iface.lacp
        if _lacp_info:
            _table.append(['LACP Sys Priority:', self.iface.lacp.sys_priority])
            _table.append(['LACP Rate:', self.lacp_rate()])
            _table.append(['LACP Rate:', self.lacp_bypass()])
        return tabulate(_table, _header) + self.new_line()

    def bondmem_details(self):
        """
        print out table with bond member summary info for netshow interface [ifacename]
        for bond interface
        """
        _header = ['', 'Port', 'Speed',
                   'TX', 'RX', 'Err', 'Link Failures']
        _table = []
        _bondmembers = self.iface.members
        if len(_bondmembers) == 0:
            return 'Has No Bond Members'
        first_member = next(iter(self.iface.members))
        if _bondmembers[first_member].counters.tx:
            _header = ['', 'Port', 'Speed',
                       'TX', 'RX', 'Err', 'Link Failures']

            for _bondmem in _bondmembers.values():
                _printbondmem = PrintBondMember(_bondmem)
                _table.append([_printbondmem.linkstate,
                               "%s(%s)" % (_printbondmem.name,
                                           self.abbrev_bondstate(_bondmem)),
                               _printbondmem.speed,
                               _bondmem.counters.total_tx,
                               _bondmem.counters.total_rx,
                               _bondmem.counters.total_err,
                               _bondmem.linkfailures])
            return tabulate(_table, _header, floatfmt='.0f') + self.new_line()
        else:
            _header = ['', 'Port', 'Speed',
                       'Link Failures']

            for _bondmem in _bondmembers.values():
                _printbondmem = PrintBondMember(_bondmem)
                _table.append([_printbondmem.linkstate,
                               "%s(%s)" % (_printbondmem.name,
                                           self.abbrev_bondstate(_bondmem)),
                               _printbondmem.speed,
                               _bondmem.linkfailures])
            return tabulate(_table, _header, floatfmt='.0f') + self.new_line()

    def clag_summary(self):
        """
        :return: clag summary details for 'netshow interface' for all ints
        """
        if self.iface.clag_enable == '1':
            return 'In CLAG'

        return ''

    @property
    def summary(self):
        """
        :return: summary info for bonds for 'netshow interfaces'
        """
        _arr = []
        _arr.append(self.print_bondmems())
        if self.iface.is_l3():
            _arr.append(', '.join(self.iface.ip_address.allentries))
        elif self.iface.is_trunk():
            _arr += self.trunk_summary()
        elif self.iface.is_access():
            _arr += self.access_summary()
        _in_clag = self.clag_summary()
        if _in_clag:
            _clag_output = "(%s)" % (_in_clag)
            _arr += [_clag_output]
        return _arr

    def cli_output(self):
        """
        point to linux provider PrintBond cli_output function
        """
        return linux_print_bond.PrintBond.cli_output(self)
