# http://pylint-messages.wikidot.com/all-codes
"""
This module collects basic counters from the device
"""
from netshowlib.linux import common
import os


def cacheinfo():
    """
    Cacheinfo function for Basic counter information
    Basic counters are defined in the IfaceCounters class.
    :return: hash of basic interface counters
    """
    mapping = {}
    for ifacename in common.portname_list():
        if common.has_stats(ifacename):
            iface_count = IfaceCounters(ifacename)
            mapping[ifacename] = {
                'rx': {
                    'packets': iface_count.rx_packets(),
                    'bytes': iface_count.rx_bytes(),
                    'errors': iface_count.rx_errors()
                },
                'tx': {
                    'packets': iface_count.tx_packets(),
                    'bytes': iface_count.tx_bytes(),
                    'errors': iface_count.tx_errors()
                }
            }
    return mapping


def gen_method(stat):
    """ generate methods that collect various stats from /sys/class/net
    """
    def _method(self):
        return common.read_from_sys(os.path.join('statistics', stat),
                                    self.name, True)
    return _method


class IfaceCounters(object):
    """
    Basic Iface counters
    Dynamically generated Functions:
        rx_bytes() - all receive bytes
        rx_packets() - all receive packets
        tx_bytes() - all transmitted bytes
        tx_packets() - all transmitted packets
        tx_errors() - all transmit errors
        rx_errors() - all receive errors
    """
    def __init__(self, name, cache=None):
        self.name = name
        self.tx = {
            'unicast': 0,
            'broadcast': 0,
            'multicast': 0,
            'errors': 0
        }
        self.rx = {
            'unicast': 0,
            'broadcast': 0,
            'multicast': 0,
            'errors': 0
        }

        self.cache = cache
        self.name = name

    @property
    def total_tx(self):
        """
        :return: total tx counters
        """
        if self.tx:
          return self.tx.get('unicast') + \
              self.tx.get('multicast') + \
              self.tx.get('broadcast')

    @property
    def total_rx(self):
        """
        :return: total rx counters
        """
        if self.rx:
          return self.rx.get('unicast') + \
              self.rx.get('multicast') + \
              self.rx.get('broadcast')

    @property
    def total_err(self):
        """
        :return: total  error counters
        """
        if self.rx and self.tx:
          return self.rx.get('errors') + \
              self.tx.get('errors')
        if self.rx:
            return self.rx.get('errors')
        if self.tx:
            return self.tx.get('errors')

    @property
    def all(self):
        """
        :return: all both tx and rx counters as a hash
        """
        return {'tx': self.tx, 'rx': self.rx}

    def read_from_sys(self, attr, oneline=True):
        """
        reads an attribute found in the
        ``/sys/class/net/[iface_name]/statistics`` directory
        """
        return common.read_from_sys(attr, self.name, oneline)

    def run(self):
        if self.cache:
            return self.cache.counters.get(self.name)
        else:
            self.cache = cacheinfo()
            return self.cache.get(self.name)


for stat in ['rx_bytes', 'tx_bytes', 'rx_packets', 'tx_packets',
             'tx_errors', 'rx_errors']:
    _method = gen_method(stat)
    setattr(IfaceCounters, stat, _method)
